@@ -3,10 +3,11 @@ Created on Tue Jan 25 16:34:41 2022
 
 @authors: Andrea Bassi @Polimi, Mark Neil @ImperialCollege
 """
-from napari_sim_processor.widget_settings import Setting, Combo_box
-from napari_sim_processor.baseSimProcessor import pytorch, cupy
-from napari_sim_processor.hexSimProcessor import HexSimProcessor
-from napari_sim_processor.convSimProcessor import ConvSimProcessor
+from .widget_settings import Setting, Combo_box
+from .processors.baseSimProcessor import pytorch, cupy
+from .processors.hexSimProcessor import HexSimProcessor
+from .processors.raSimProcessor import RaSimProcessor
+from .processors.convSimProcessor import ConvSimProcessor
 import napari
 from qtpy.QtWidgets import QVBoxLayout, QSplitter, QHBoxLayout, QWidget, QPushButton, QLineEdit
 from qtpy.QtCore import Qt
@@ -14,7 +15,7 @@ from napari.layers import Image
 import numpy as np
 from napari.qt.threading import thread_worker
 from magicgui.widgets import FunctionGui
-from magicgui import magicgui, magic_factory
+from magicgui import magic_factory
 import warnings
 import time
 from superqt.utils import qthrottled
@@ -24,6 +25,7 @@ from enum  import Enum
 class Sim_modes(Enum):
     SIM = 1
     HEXSIM = 2
+    RASIM = 3
 
 
 class Accel(Enum):
@@ -122,6 +124,11 @@ def reshape(viewer: napari.Viewer,
         viewer.dims.set_point(axis=[0,1,2], value=[0,0,0]) #raises ValueError in napari versions <0.4.13 
 
 
+@magic_factory
+def choose_layer(image: Image):
+        pass #TODO: substitute with a qtwidget without magic functions
+
+
 class SimAnalysis(QWidget):
     '''
     A Napari plugin for the reconstruction of Structured Illumination microscopy (SIM) data with GPU acceleration (with pytorch, if installed).
@@ -161,7 +168,12 @@ class SimAnalysis(QWidget):
         right_layout = QVBoxLayout()
         bottom_layout.addLayout(right_layout)
         # Fill top layout
-        self.add_magic_function(self.select_layer, top_layout)
+        self.choose_layer_widget = choose_layer()
+        self.choose_layer_widget.call_button.visible = False
+        self.add_magic_function(self.choose_layer_widget, top_layout)
+        select_button = QPushButton('Select image layer')
+        select_button.clicked.connect(self.select_layer)
+        top_layout.addWidget(select_button)
         # Fill bottom-left layout
         self.sim_mode = Combo_box(name = 'Mode', choices = Sim_modes, layout=left_layout,
                                   write_function = self.reset_processor)
@@ -253,27 +265,27 @@ class SimAnalysis(QWidget):
         self.messageBox = QLineEdit()
         layout.addWidget(self.messageBox, stretch=True)
         self.messageBox.setText('Messages')
-    
         
-    def add_magic_function(self, function, _layout):
-        self.viewer.layers.events.inserted.connect(function.reset_choices)
-        self.viewer.layers.events.removed.connect(function.reset_choices)
-        _layout.addWidget(function.native)
         
-    
-    @magicgui(call_button='Select image layer')    
+    def add_magic_function(self, widget, _layout):
+        self.viewer.layers.events.inserted.connect(widget.reset_choices)
+        self.viewer.layers.events.removed.connect(widget.reset_choices)
+        _layout.addWidget(widget.native)
+
+
     def select_layer(self, image: Image):
         '''
-        Selects a Image layer after chaking that it contains raw sim data organized
+        Selects a Image layer after checking that it contains raw sim data organized
         as a 5D stack (angle,phase,z,y,x).
         Stores the name of the image in self.imageRaw_name, which is used frequently in the other methods.
         Parameters
         ----------
         image : napari.layers.Image
             The image layer to process, it contains the raw data 
-        '''      
+        '''
+        image = self.choose_layer_widget.image.value
         if not isinstance(image, Image):
-            return
+            raise(KeyError('Please select a image stack'))
         if hasattr(self,'imageRaw_name'):
             delattr(self,'imageRaw_name')
         data = image.data
@@ -281,8 +293,8 @@ class SimAnalysis(QWidget):
             raise(KeyError('Please select a 5D(angle,phase,z,y,x) stack'))
         self.imageRaw_name = image.name
         sa,sp,sz,sy,sx = image.data.shape
-        if not sy == sx:
-            raise(KeyError('Non-square images are not supported'))
+        # if not sy == sx:
+        #     raise(KeyError('Non-square images are not supported'))
         self.angles_number.val = sa
         self.phases_number.val = sp
         self.viewer.dims.axis_labels = ["angle", "phase", "z", "y","x"]
@@ -339,7 +351,7 @@ class SimAnalysis(QWidget):
             if self.step_changed(0,2,3,4):
                 self.setReconstructor()
             delta_t = time.time() - t0
-            self.on_step_change.set_timeout(delta_t)
+            self.on_step_change.set_timeout(int(delta_t)+1)
 
 
     def step_changed(self, *step_indexes):
@@ -379,7 +391,8 @@ class SimAnalysis(QWidget):
                                             name = im_name,
                                             scale = scale,
                                             colormap = colormap,
-                                            interpolation = 'bilinear')
+                                            )
+            layer.interpolation2d = 'spline36'
         self.center_stack(image_values)
         # self.move_layer_to_top(layer)
         if kwargs.get('autoscale') is True:
@@ -490,13 +503,16 @@ class SimAnalysis(QWidget):
                 self.start_sim_processor()
             else:
                 if self.sim_mode.current_data == Sim_modes.HEXSIM.value:  
-                    self.h = HexSimProcessor()  
+                    self.h = HexSimProcessor(self.phases_number.val)
                     k_shape = (3,1)
                 elif self.sim_mode.current_data == Sim_modes.SIM.value and self.phases_number.val >= 3 and self.angles_number.val > 0:
                     self.h = ConvSimProcessor(angleSteps=self.angles_number.val,
                                               phaseSteps=self.phases_number.val)
-                    k_shape = (self.angles_number.val,1)   
-                else: 
+                    k_shape = (self.angles_number.val,1)
+                elif self.sim_mode.current_data == Sim_modes.RASIM.value:
+                    self.h = RaSimProcessor(self.phases_number.val)
+                    k_shape = (2, 1)
+                else:
                     raise(ValueError("Invalid phases or angles number"))
                 self.carrier_idx.set_min_max(0,k_shape[0]-1) # TODO connect carrier idx to angle if Sim_mode == SIM
                 self.h.debug = False
@@ -628,18 +644,25 @@ class SimAnalysis(QWidget):
         if self.is_image_in_layers() :
             name = f'carrier_{self.imageRaw_name}'
             if self.showCarrier.val and self.isCalibrated:
-                N = self.h.N
-                cutoff, dk = self.calculate_kr(N)
+                Nx = self.h.Nx
+                Ny = self.h.Ny
+                cutoff, dkx = self.calculate_kr(Nx)
+                cutoff, dky = self.calculate_kr(Ny)
                 kxs = self.h.kx
                 kys = self.h.ky
                 pc = np.zeros((len(kxs),2))
                 radii = []
+                colors = []
                 for idx, (kx,ky) in enumerate(zip(kxs,kys)):
-                    pc[idx,0] = ky[0] / dk + N/2
-                    pc[idx,1] = kx[0] / dk + N/2
-                    radii.append(self.h.N // 30) # radius of the displayed circle
-                layer=self.add_circles(pc, radii, name, color='red')
-                self.move_layer_to_top(layer) 
+                    pc[idx,0] = ky[0] / dky + Ny/2
+                    pc[idx,1] = kx[0] / dkx + Nx/2
+                    radii.append(((Nx + Ny) // 60, (Nx + Ny) // 60)) # radius of the displayed circle
+                    if idx == self.carrier_idx.val:
+                        colors.append('red')
+                    else:
+                        colors.append('yellow')
+                layer=self.add_circles(pc, radii, name, color=colors)
+                self.move_layer_to_top(layer)
                 # kr = np.sqrt(kxs**2+kys**2)
                 # print('Carrier magnitude / cut off:', *kr/cutoff*dk)
             elif name in self.viewer.layers:
@@ -657,14 +680,17 @@ class SimAnalysis(QWidget):
             radii = []
             colors = []
             if self.showEta.val:
-                N = self.h.N
-                cutoff, dk   = self.calculate_kr(N)  
-                eta_radius = 1.9 * self.h.eta * cutoff
-                centres.append(np.array([N/2,N/2]))
-                radii.append(eta_radius)
+                Nx = self.h.Nx
+                Ny = self.h.Ny
+                cutoffx, dkx   = self.calculate_kr(Nx)
+                cutoffy, dky   = self.calculate_kr(Ny)
+                eta_radiusx = 1.9 * self.h.eta * cutoffx
+                eta_radiusy = 1.9 * self.h.eta * cutoffy
+                centres.append(np.array([Ny/2,Nx/2]))
+                radii.append((eta_radiusy, eta_radiusx))
                 colors.append('green')
-                centres.append(np.array([N/2,N/2]))
-                radii.append(2 * cutoff)
+                centres.append(np.array([Ny/2,Nx/2]))
+                radii.append((2 * cutoffy, 2 * cutoffx))
                 colors.append('blue')
                 layer=self.add_circles(centres, radii, shape_name=name, color=colors)
                 self.move_layer_to_top(layer)
@@ -693,10 +719,10 @@ class SimAnalysis(QWidget):
         '''
         ellipses = []
         for center, radius in zip(locations, radii):
-            bbox = np.array([center + np.array([radius, radius]),
-                             center + np.array([radius, -radius]),
-                             center + np.array([-radius, -radius]),
-                             center + np.array([-radius, radius])]
+            bbox = np.array([center + np.array([radius[0], radius[1]]),
+                             center + np.array([radius[0], -radius[1]]),
+                             center + np.array([-radius[0], -radius[1]]),
+                             center + np.array([-radius[0], radius[1]])]
                             )
             ellipses.append(bbox)
         
@@ -884,32 +910,39 @@ class SimAnalysis(QWidget):
         assert self.isCalibrated, 'SIM processor not calibrated, unable to show phases'
         if self.sim_mode.current_data == Sim_modes.HEXSIM.value:
             self.find_hexsim_phaseshifts()
+        if self.sim_mode.current_data == Sim_modes.RASIM.value:
+            self.find_hexsim_phaseshifts()
         elif self.sim_mode.current_data == Sim_modes.SIM.value:
             self.find_sim_phaseshifts()
         
         
-    def find_hexsim_phaseshifts(self):   
-        phaseshift = np.zeros((7,3))
-        expected_phase = np.zeros((7,3))
-        error = np.zeros((7,3))
+    def find_hexsim_phaseshifts(self):
+        nbands = self.h._nbands
+        nsteps = self.h._nsteps
+        print(nbands)
+        phaseshift = np.zeros((nsteps,nbands))
+        expected_phase = np.zeros((nsteps,nbands))
+        error = np.zeros((nsteps,nbands))
         stack = self.get_current_ap_stack()
         sa,sp,sy,sx = stack.shape
         img = stack.reshape(sa*sp, sy, sx) 
-        for i in range (3):
+        for i in range (nbands):
             if self.proc.current_data == Accel.USE_TORCH.value:
                 phase, _ = self.h.find_phase_pytorch(self.h.kx[i], self.h.ky[i], img)
             elif self.proc.current_data == Accel.USE_CUPY.value:
                 phase, _ = self.h.find_phase_cupy(self.h.kx[i], self.h.ky[i], img)
             else:
                 phase, _ = self.h.find_phase(self.h.kx[i], self.h.ky[i], img)
-            expected_phase[:,i] = np.arange(7) * 2*(i+1) * np.pi / 7
+            expected_phase[:,i] = np.arange(nsteps) * 2*(i+1) * np.pi / nsteps
             phaseshift[:,i] = np.unwrap(phase - phase[0] - expected_phase[:,i]) + expected_phase[:,i]
         error = phaseshift-expected_phase
         data_to_plot = [expected_phase, phaseshift, error]
         symbols = ['.','o','|']
         legend = ['expected', 'measured', 'error']
         self.plot_with_plt(data_to_plot, legend, symbols,
-                                xlabel = 'step', ylabel = 'phase (rad)', vmax = 6*np.pi)
+                                xlabel = 'step', ylabel = 'phase (rad)',
+                                vmax = np.amax(expected_phase),
+                                ticks_num = nsteps)
             
     
     def find_sim_phaseshifts(self):   
@@ -929,18 +962,22 @@ class SimAnalysis(QWidget):
             phase = np.unwrap(phase)
             phase = phase.reshape(sa,sp).T
             expected_phase[:,angle_idx] = np.arange(sp) * 2*np.pi / sp
-            phaseshift= phase-phase[0,:]
+            phaseshift = phase-phase[0,:]
             error = phaseshift-expected_phase      
             data_to_plot = [expected_phase[:,angle_idx], phaseshift[:,angle_idx], error[:,angle_idx]]
             symbols = ['.','o','|']
             legend = ['expected', 'measured', 'error']
             self.plot_with_plt(data_to_plot, legend, symbols, title = f'angle {angle_idx}',
-                                    xlabel = 'step', ylabel = 'phase (rad)', vmax = 2*np.pi)
+                                    xlabel = 'step', ylabel = 'phase (rad)',
+                                    vmax = np.amax(expected_phase),
+                                    ticks_num = sp)
                              
     
     def plot_with_plt(self, data_list, legend, symbols,
                       xlabel = 'step', ylabel = 'phase',
-                      vmax = 2*np.pi, title = ''):
+                      vmax = 2*np.pi,
+                      ticks_num = 3 ,
+                      title = ''):
         import matplotlib.pyplot as plt
         char_size = 10
         plt.rc('font', family='calibri', size=char_size)
@@ -966,23 +1003,10 @@ class SimAnalysis(QWidget):
         ax.yaxis.set_tick_params(labelsize=char_size*0.75)
         ax.legend(legend, loc='best', frameon = False, fontsize=char_size*0.8)
         ax.grid(True, which='major', axis='both', alpha=0.2)
-        vales_num = s[0]
-        ticks = np.linspace(0, vmax*(vales_num-1)/vales_num, 2*vales_num-1 )
+        ticks = np.linspace(0, vmax, ticks_num)
         ax.set_yticks(ticks)
         fig.tight_layout()
         plt.show(block=False)
         plt.rcParams.update(plt.rcParamsDefault)
 
 
-if __name__ == '__main__':
-
-    import napari
-    viewer = napari.Viewer()
-    widget = SimAnalysis(viewer)
-    my_reshape_widget = reshape()    
-    viewer.window.add_dock_widget(my_reshape_widget, name = 'Reshape stack', add_vertical_stretch = True)
-    viewer.window.add_dock_widget(widget,
-                                  name = 'Sim analyzer @Polimi',
-                                  add_vertical_stretch = True)
-    napari.run() 
-     
\ No newline at end of file
